@@ -42,17 +42,6 @@ extern "C"
 {
 #endif
 
-#define KCAPI_MAJVERSION 0  /* API / ABI incompatible changes, functional
-			     * changes that require consumer to be updated
-			     * (as long as this number is zero, the API is
-			     * not considered stable and can change without
-			     * a bump of the major version) */
-#define KCAPI_MINVERSION 13 /* API compatible, ABI may change, functional
-			     * enhancements only, consumer can be left
-			     * unchanged if enhancements are not considered */
-#define KCAPI_PATCHLEVEL 0  /* API / ABI compatible, no functional changes, no
-			     * enhancements, bug fixes only */
-
 /* remove once in if_alg.h */
 #ifndef ALG_SET_AEAD_ASSOCLEN
 #define ALG_SET_AEAD_ASSOCLEN		4
@@ -63,6 +52,13 @@ extern "C"
 #ifndef ALG_SET_PUBKEY
 #define ALG_SET_PUBKEY			6
 #endif
+#ifndef ALG_SET_DH_PARAMETERS
+#define ALG_SET_DH_PARAMETERS		7
+#endif
+#ifndef ALG_SET_ECDH_CURVE
+#define ALG_SET_ECDH_CURVE		8
+#endif
+
 
 #ifndef ALG_OP_SIGN
 #define ALG_OP_SIGN			2
@@ -71,6 +67,13 @@ extern "C"
 #define ALG_OP_VERIFY			3
 #endif
 
+#ifndef ALG_OP_KEYGEN
+#define ALG_OP_KEYGEN			4
+#endif
+#ifndef ALG_OP_SSGEN
+#define ALG_OP_SSGEN			5
+#endif
+
 /* remove once in socket.h */
 #ifndef AF_ALG
 #define AF_ALG 38
@@ -169,7 +172,7 @@ struct kcapi_aead_data {
  * @cio: Active concurrent IOCBs
  */
 struct kcapi_aio {
-	unsigned int skcipher_aio_disable:1;
+	unsigned int disable:1;
 	int efd;
 	aio_context_t aio_ctx;
 	uint32_t completed_reads;
@@ -178,7 +181,14 @@ struct kcapi_aio {
 };
 
 struct kcapi_flags {
-	int newaeadif:1;
+	/*
+	 * New AEAD interface introduced with 4.9.0 to only require a tag
+	 * if it is required as input or output.
+	 */
+	unsigned int newtag:1;
+
+	/* AF_ALG interfaces changed to process more pages concurrently. */
+	uint32_t alg_max_pages;
 };
 
 struct kcapi_sys {
@@ -198,8 +208,8 @@ struct kcapi_sys {
  */
 struct kcapi_handle {
 	int tfmfd;
-	int opfd;
 	int pipes[2];
+	int opfd;
 	uint32_t processed_sg;
 	struct kcapi_sys sysinfo;
 	struct kcapi_cipher_data cipher;
@@ -216,19 +226,69 @@ struct kcapi_handle {
 int kcapi_verbosity_level;
 void kcapi_dolog(int severity, const char *fmt, ...);
 
-int32_t _kcapi_common_send_meta(struct kcapi_handle *handle, struct iovec *iov,
-				uint32_t iovlen, uint32_t enc, uint32_t flags);
-int32_t _kcapi_common_vmsplice_iov(struct kcapi_handle *handle,
+int32_t _kcapi_common_send_meta_fd(struct kcapi_handle *handle, int *fdptr,
+				   struct iovec *iov, uint32_t iovlen,
+				   uint32_t enc, uint32_t flags);
+inline int32_t _kcapi_common_send_meta(struct kcapi_handle *handle,
+				       struct iovec *iov, uint32_t iovlen,
+				       uint32_t enc, uint32_t flags)
+{
+	return _kcapi_common_send_meta_fd(handle, &handle->opfd, iov, iovlen,
+					  enc, flags);
+}
+
+int32_t _kcapi_common_vmsplice_iov_fd(struct kcapi_handle *handle, int *fdptr,
+				      struct iovec *iov, unsigned long iovlen,
+				      uint32_t flags);
+inline int32_t _kcapi_common_vmsplice_iov(struct kcapi_handle *handle,
 				   struct iovec *iov, unsigned long iovlen,
+				   uint32_t flags)
+{
+	return _kcapi_common_vmsplice_iov_fd(handle, &handle->opfd, iov, iovlen,
+					     flags);
+}
+
+int32_t _kcapi_common_send_data_fd(struct kcapi_handle *handle, int *fdprt,
+				   struct iovec *iov, uint32_t iovlen,
 				   uint32_t flags);
-int32_t _kcapi_common_send_data(struct kcapi_handle *handle, struct iovec *iov,
-				uint32_t iovlen, uint32_t flags);
-int32_t _kcapi_common_recv_data(struct kcapi_handle *handle, struct iovec *iov,
-				uint32_t iovlen);
-int _kcapi_common_accept(struct kcapi_handle *handle);
-int32_t _kcapi_common_vmsplice_chunk(struct kcapi_handle *handle,
-				     const uint8_t *in, uint32_t inlen,
-				     uint32_t flags);
+inline int32_t _kcapi_common_send_data(struct kcapi_handle *handle,
+				       struct iovec *iov, uint32_t iovlen,
+				       uint32_t flags)
+{
+	return _kcapi_common_send_data_fd(handle, &handle->opfd, iov, iovlen,
+					  flags);
+}
+
+int32_t _kcapi_common_recv_data_fd(struct kcapi_handle *handle, int *fdptr,
+				   struct iovec *iov, uint32_t iovlen);
+inline int32_t _kcapi_common_recv_data(struct kcapi_handle *handle,
+				       struct iovec *iov, uint32_t iovlen)
+{
+	return _kcapi_common_recv_data_fd(handle, &handle->opfd, iov, iovlen);
+}
+
+int32_t _kcapi_common_read_data_fd(struct kcapi_handle *handle, int *fdptr,
+				   uint8_t *out, uint32_t outlen);
+inline int32_t _kcapi_common_read_data(struct kcapi_handle *handle,
+				       uint8_t *out, uint32_t outlen)
+{
+	return _kcapi_common_read_data_fd(handle, &handle->opfd, out, outlen);
+}
+
+int _kcapi_common_accept(struct kcapi_handle *handle, int *fdptr);
+int _kcapi_common_close(struct kcapi_handle *handle, int fdptr);
+
+int32_t _kcapi_common_vmsplice_chunk_fd(struct kcapi_handle *handle, int *fdptr,
+				        const uint8_t *in, uint32_t inlen,
+				        uint32_t flags);
+inline int32_t _kcapi_common_vmsplice_chunk(struct kcapi_handle *handle,
+					    const uint8_t *in, uint32_t inlen,
+					    uint32_t flags)
+{
+	return _kcapi_common_vmsplice_chunk_fd(handle, &handle->opfd, in,
+					       inlen, flags);
+}
+
 
 int _kcapi_handle_init(struct kcapi_handle **caller, const char *type,
 		       const char *ciphername, uint32_t flags);
@@ -246,6 +306,20 @@ int32_t _kcapi_cipher_crypt_aio(struct kcapi_handle *handle,
 				struct iovec *iniov, struct iovec *outiov,
 				uint32_t iovlen, int access, int enc);
 
+int _kcapi_aio_send_iov(struct kcapi_handle *handle, struct iovec *iov,
+			uint32_t iovlen, int access, int enc);
+
+int32_t _kcapi_aio_read_iov_fd(struct kcapi_handle *handle, int *fdptr,
+			       struct iovec *iov, uint32_t iovlen);
+inline int32_t _kcapi_aio_read_iov(struct kcapi_handle *handle,
+				   struct iovec *iov, uint32_t iovlen)
+{
+	return _kcapi_aio_read_iov_fd(handle, &handle->opfd, iov, iovlen);
+}
+
+int32_t _kcapi_aio_read_all(struct kcapi_handle *handle, uint32_t toread,
+			    struct timespec *timeout);
+
 /************************************************************
  * Declarations for system calls
  ************************************************************/
