@@ -50,12 +50,10 @@ int32_t kcapi_cipher_encrypt(struct kcapi_handle *handle,
 	uint32_t bs = handle->info.blocksize;
 
 	/* require properly sized output data size */
-	if (outlen < ((inlen + bs - 1) / bs * bs)) {
-		kcapi_dolog(LOG_ERR,
+	if (outlen < ((inlen + bs - 1) / bs * bs))
+		kcapi_dolog(LOG_WARN,
 			    "Symmetric Encryption: Ciphertext buffer (%lu) is not plaintext buffer (%lu) rounded up to multiple of block size %u",
 			    (unsigned long) outlen, (unsigned long)inlen, bs);
-		return -EINVAL;
-	}
 
 	handle->cipher.iv = iv;
 	return _kcapi_cipher_crypt_chunk(handle, in, inlen, out, outlen, access,
@@ -70,25 +68,12 @@ _kcapi_cipher_encrypt_aio_fallback(struct kcapi_handle *handle,
 				 struct iovec *iniov, struct iovec *outiov,
 				 uint32_t iovlen, const uint8_t *iv)
 {
-	int32_t ret = 0;
-	uint32_t i;
+	int32_t rc = kcapi_cipher_stream_init_enc(handle, iv, iniov, iovlen);
 
-	/* every IOVEC for AIO is an independent cipher invocation */
-	for (i = 0; i < iovlen; i++) {
-		int32_t rc = kcapi_cipher_stream_init_enc(handle, iv, &iniov[i],
-							  1);
+	if (rc < 0)
+		return rc;
 
-		if (rc < 0)
-			return rc;
-
-		rc = kcapi_cipher_stream_op(handle, &outiov[i], 1);
-		if (rc < 0)
-			return rc;
-
-		ret += rc;
-	}
-
-	return ret;
+	return kcapi_cipher_stream_op(handle, outiov, iovlen);
 }
 
 DSO_PUBLIC
@@ -99,14 +84,18 @@ int32_t kcapi_cipher_encrypt_aio(struct kcapi_handle *handle,
 	int32_t ret;
 
 	handle->cipher.iv = iv;
-	ret = _kcapi_cipher_crypt_aio(handle, iniov, outiov, iovlen, access,
-				      ALG_OP_ENCRYPT);
 
-	if (ret == -EOPNOTSUPP)
-		return _kcapi_cipher_encrypt_aio_fallback(handle, iniov, outiov,
-							  iovlen, iv);
+	if (handle->aio.disable) {
+		ret = _kcapi_cipher_crypt_aio(handle, iniov, outiov, iovlen,
+					      access, ALG_OP_ENCRYPT);
 
-	return ret;
+		if (ret != -EOPNOTSUPP)
+			return ret;
+	}
+
+	/* The kernel's AIO interface is broken. */
+	return _kcapi_cipher_encrypt_aio_fallback(handle, iniov, outiov,
+						  iovlen, iv);
 }
 
 DSO_PUBLIC
@@ -116,19 +105,15 @@ int32_t kcapi_cipher_decrypt(struct kcapi_handle *handle,
 			     uint8_t *out, uint32_t outlen, int access)
 {
 	/* require properly sized output data size */
-	if (inlen % handle->info.blocksize) {
-		kcapi_dolog(LOG_ERR,
+	if (inlen % handle->info.blocksize)
+		kcapi_dolog(LOG_WARN,
 			    "Symmetric Decryption: Ciphertext buffer is not multiple of block size %u",
 			    handle->info.blocksize);
-		return -EINVAL;
-	}
 
-	if (outlen < inlen) {
-		kcapi_dolog(LOG_ERR,
+	if (outlen < inlen)
+		kcapi_dolog(LOG_WARN,
 			    "Symmetric Decryption: Plaintext buffer (%lu) is smaller as ciphertext buffer (%lu)",
 			    (unsigned long)outlen, (unsigned long)inlen);
-		return -EINVAL;
-	}
 
 	handle->cipher.iv = iv;
 	return _kcapi_cipher_crypt_chunk(handle, in, inlen, out, outlen, access,
@@ -143,25 +128,12 @@ _kcapi_cipher_decrypt_aio_fallback(struct kcapi_handle *handle,
 				   struct iovec *iniov, struct iovec *outiov,
 				   uint32_t iovlen, const uint8_t *iv)
 {
-	int32_t ret = 0;
-	uint32_t i;
+	int32_t rc = kcapi_cipher_stream_init_dec(handle, iv, iniov, iovlen);
 
-	/* every IOVEC for AIO is an independent cipher invocation */
-	for (i = 0; i < iovlen; i++) {
-		int32_t rc = kcapi_cipher_stream_init_dec(handle, iv, &iniov[i],
-							  1);
+	if (rc < 0)
+		return rc;
 
-		if (rc < 0)
-			return rc;
-
-		rc = kcapi_cipher_stream_op(handle, &outiov[i], 1);
-		if (rc < 0)
-			return rc;
-
-		ret += rc;
-	}
-
-	return ret;
+	return kcapi_cipher_stream_op(handle, outiov, iovlen);
 }
 
 DSO_PUBLIC
@@ -172,14 +144,18 @@ int32_t kcapi_cipher_decrypt_aio(struct kcapi_handle *handle,
 	int32_t ret;
 
 	handle->cipher.iv = iv;
-	ret = _kcapi_cipher_crypt_aio(handle, iniov, outiov, iovlen, access,
-				      ALG_OP_DECRYPT);
 
-	if (ret == -EOPNOTSUPP)
-		return _kcapi_cipher_decrypt_aio_fallback(handle, iniov, outiov,
-							  iovlen, iv);
+	if (handle->aio.disable) {
+		ret = _kcapi_cipher_crypt_aio(handle, iniov, outiov, iovlen,
+					      access, ALG_OP_DECRYPT);
 
-	return ret;
+		if (ret != -EOPNOTSUPP)
+			return ret;
+	}
+
+	/* The kernel's AIO interface is broken. */
+	return _kcapi_cipher_decrypt_aio_fallback(handle, iniov, outiov,
+						  iovlen, iv);
 }
 
 DSO_PUBLIC
@@ -206,7 +182,7 @@ DSO_PUBLIC
 int32_t kcapi_cipher_stream_update(struct kcapi_handle *handle,
 				   struct iovec *iov, uint32_t iovlen)
 {
-	if (handle->processed_sg <= ALG_MAX_PAGES)
+	if (handle->processed_sg <= handle->flags.alg_max_pages)
 		return _kcapi_common_vmsplice_iov(handle, iov, iovlen,
 						  SPLICE_F_MORE);
 	else
