@@ -28,6 +28,9 @@ extern "C"
 {
 #endif
 
+#define DSO_DEPRECATED(x) \
+	__attribute__ ((deprecated ("API deprecated with library version " x)))
+
 /*
  * Flags for the encrypt / decrypt operations
  * 
@@ -47,7 +50,7 @@ extern "C"
  * 
  * @KCAPI_INIT_AIO Handle uses AIO kernel interface if available
  */
-#define KCAPI_INIT_AIO	0x1
+#define KCAPI_INIT_AIO			(1<<0)
 
 /*
  * Opaque cipher handle
@@ -103,7 +106,10 @@ void kcapi_cipher_destroy(struct kcapi_handle *handle);
  * After the caller provided the key, the caller may securely destroy the key
  * as it is now maintained by the kernel.
  *
- * @return 0 upon success;
+ * @return 0 upon success (in case of an akcipher handle, a positive integer
+ *	   is returned that denominates the maximum output size of the
+ *	   cryptographic operation -- this value must be used as the size
+ *	   of the output buffer for one cryptographic operation);
  *	   a negative errno-style error code if an error occurred
  */
 int kcapi_cipher_setkey(struct kcapi_handle *handle,
@@ -571,6 +577,7 @@ int32_t kcapi_aead_encrypt_aio(struct kcapi_handle *handle, struct iovec *iniov,
  * provides the consumer with the length of the tag and the length of the
  * ciphertext.
  */
+DSO_DEPRECATED("0.13.0")
 void kcapi_aead_getdata(struct kcapi_handle *handle,
 			uint8_t *encdata, uint32_t encdatalen,
 			uint8_t **aad, uint32_t *aadlen,
@@ -931,6 +938,7 @@ uint32_t kcapi_aead_authsize(struct kcapi_handle *handle);
  *
  * @return minimum size of output data length in bytes
  */
+DSO_DEPRECATED("0.13.0")
 uint32_t kcapi_aead_outbuflen(struct kcapi_handle *handle, uint32_t inlen,
 			      uint32_t assoclen, uint32_t taglen);
 
@@ -1150,7 +1158,7 @@ uint32_t kcapi_md_blocksize(struct kcapi_handle *handle);
  * @handle: [out] cipher handle filled during the call
  * @ciphername: [in] kernel crypto API cipher name as specified in
  *	/proc/crypto
- * @flags: [in] flags specifying the type of cipher handle
+ * @flags: [in] flags specifying the type of cipher handle (unused for RNG)
  *
  * This function provides the initialization of a random number generator handle
  * and establishes the connection to the kernel.
@@ -1208,6 +1216,16 @@ int kcapi_rng_seed(struct kcapi_handle *handle, uint8_t *seed,
 int32_t kcapi_rng_generate(struct kcapi_handle *handle,
 			   uint8_t *buffer, uint32_t len);
 
+/**
+ * kcapi_rng_seedsize() - return required seed size of DRNG
+ *
+ * @handle: [in] cipher handle
+ *
+ * @return > 0 specifying the block size;
+ *	   0 on error
+ */
+uint32_t kcapi_rng_seedsize(struct kcapi_handle *handle);
+
 /**
  * DOC: Common API
  *
@@ -1215,6 +1233,7 @@ int32_t kcapi_rng_generate(struct kcapi_handle *handle,
  */
 
 enum kcapi_verbosity {
+	LOG_NONE,
 	LOG_ERR,
 	LOG_WARN,
 	LOG_VERBOSE,
@@ -1414,6 +1433,32 @@ int32_t kcapi_akcipher_encrypt(struct kcapi_handle *handle,
 			       const uint8_t *in, uint32_t inlen,
 			       uint8_t *out, uint32_t outlen, int access);
 
+/**
+ * kcapi_akcipher_encrypt_aio() - encrypt data (asynchronous one shot)
+ *
+ * @handle: [in] cipher handle
+ * @iniov: [in] head of scatter-gather list array holding the plaintext
+ * @outiov: [out] head of scatter-gather list of the destination buffers filled
+ *	with ciphertext
+ * @iovlen: [in] number of scatter-gather list entries
+ * @access: [in] kernel access type (KCAPI_ACCESS_HEURISTIC - use internal
+ *	heuristic for  fastest kernel access; KCAPI_ACCESS_VMSPLICE - use
+ *	vmsplice access; KCAPI_ACCESS_SENDMSG - sendmsg access)
+ *
+ * The individual scatter-gather list entries are processed with
+ * separate invocations of the the given cipher.
+ *
+ * The memory should be aligned at the page boundary using
+ * posix_memalign(sysconf(_SC_PAGESIZE)), If it is not aligned at the page
+ * boundary, the vmsplice call may not send all data to the kernel.
+ *
+ * @return number of bytes encrypted upon success;
+ *	   a negative errno-style error code if an error occurred
+ */
+int32_t kcapi_akcipher_encrypt_aio(struct kcapi_handle *handle,
+				   struct iovec *iniov, struct iovec *outiov,
+				   uint32_t iovlen, int access);
+
 /**
  * kcapi_akcipher_decrypt() - decrypt data
  *
@@ -1444,6 +1489,32 @@ int32_t kcapi_akcipher_decrypt(struct kcapi_handle *handle,
 			       const uint8_t *in, uint32_t inlen,
 			       uint8_t *out, uint32_t outlen, int access);
 
+/**
+ * kcapi_akcipher_decrypt_aio() - decrypt data (asynchronous one shot)
+ *
+ * @handle: [in] cipher handle
+ * @iniov: [in] head of scatter-gather list array holding the plaintext
+ * @outiov: [out] head of scatter-gather list of the destination buffers filled
+ *	with ciphertext
+ * @iovlen: [in] number of scatter-gather list entries
+ * @access: [in] kernel access type (KCAPI_ACCESS_HEURISTIC - use internal
+ *	heuristic for  fastest kernel access; KCAPI_ACCESS_VMSPLICE - use
+ *	vmsplice access; KCAPI_ACCESS_SENDMSG - sendmsg access)
+ *
+ * The individual scatter-gather list entries are processed with
+ * separate invocations of the the given cipher.
+ *
+ * The memory should be aligned at the page boundary using
+ * posix_memalign(sysconf(_SC_PAGESIZE)), If it is not aligned at the page
+ * boundary, the vmsplice call may not send all data to the kernel.
+ *
+ * @return number of bytes decrypted upon success;
+ *	   a negative errno-style error code if an error occurred
+ */
+int32_t kcapi_akcipher_decrypt_aio(struct kcapi_handle *handle,
+				   struct iovec *iniov, struct iovec *outiov,
+				   uint32_t iovlen, int access);
+
 /**
  * kcapi_akcipher_sign() - signature generation
  *
@@ -1474,6 +1545,32 @@ int32_t kcapi_akcipher_sign(struct kcapi_handle *handle,
 			    const uint8_t *in, uint32_t inlen,
 			    uint8_t *out, uint32_t outlen, int access);
 
+/**
+ * kcapi_akcipher_sign_aio() - sign data (asynchronous one shot)
+ *
+ * @handle: [in] cipher handle
+ * @iniov: [in] head of scatter-gather list array holding the plaintext
+ * @outiov: [out] head of scatter-gather list of the destination buffers filled
+ *	with ciphertext
+ * @iovlen: [in] number of scatter-gather list entries
+ * @access: [in] kernel access type (KCAPI_ACCESS_HEURISTIC - use internal
+ *	heuristic for  fastest kernel access; KCAPI_ACCESS_VMSPLICE - use
+ *	vmsplice access; KCAPI_ACCESS_SENDMSG - sendmsg access)
+ *
+ * The individual scatter-gather list entries are processed with
+ * separate invocations of the the given cipher.
+ *
+ * The memory should be aligned at the page boundary using
+ * posix_memalign(sysconf(_SC_PAGESIZE)), If it is not aligned at the page
+ * boundary, the vmsplice call may not send all data to the kernel.
+ *
+ * @return number of bytes signed upon success;
+ *	   a negative errno-style error code if an error occurred
+ */
+int32_t kcapi_akcipher_sign_aio(struct kcapi_handle *handle,
+				struct iovec *iniov, struct iovec *outiov,
+				uint32_t iovlen, int access);
+
 /**
  * kcapi_akcipher_verify() - signature verification
  *
@@ -1508,6 +1605,32 @@ int32_t kcapi_akcipher_verify(struct kcapi_handle *handle,
 			      const uint8_t *in, uint32_t inlen,
 			      uint8_t *out, uint32_t outlen, int access);
 
+/**
+ * kcapi_akcipher_verify_aio() - verify data (asynchronous one shot)
+ *
+ * @handle: [in] cipher handle
+ * @iniov: [in] head of scatter-gather list array holding the plaintext
+ * @outiov: [out] head of scatter-gather list of the destination buffers filled
+ *	with ciphertext
+ * @iovlen: [in] number of scatter-gather list entries
+ * @access: [in] kernel access type (KCAPI_ACCESS_HEURISTIC - use internal
+ *	heuristic for  fastest kernel access; KCAPI_ACCESS_VMSPLICE - use
+ *	vmsplice access; KCAPI_ACCESS_SENDMSG - sendmsg access)
+ *
+ * The individual scatter-gather list entries are processed with
+ * separate invocations of the the given cipher.
+ *
+ * The memory should be aligned at the page boundary using
+ * posix_memalign(sysconf(_SC_PAGESIZE)), If it is not aligned at the page
+ * boundary, the vmsplice call may not send all data to the kernel.
+ *
+ * @return number of bytes verify upon success;
+ *	   a negative errno-style error code if an error occurred
+ */
+int32_t kcapi_akcipher_verify_aio(struct kcapi_handle *handle,
+				  struct iovec *iniov, struct iovec *outiov,
+				  uint32_t iovlen, int access);
+
 /**
  * kcapi_akcipher_stream_init_enc() - start an encryption operation (stream)
  *
@@ -1700,6 +1823,16 @@ int32_t kcapi_akcipher_stream_update_last(struct kcapi_handle *handle,
 int32_t kcapi_akcipher_stream_op(struct kcapi_handle *handle,
 			         struct iovec *iov, uint32_t iovlen);
 
+/**
+ * DOC: Key-Agreement Protocol Primitives
+ *
+ * API function calls used to invoke Diffie-Hellmand or EC-Diffie-Hellman
+ * operations.
+ */
+/* Curves IDs */
+#define ECC_CURVE_NIST_P192     0x0001
+#define ECC_CURVE_NIST_P256     0x0002
+
 /**
  * DOC: Key Derivation Functions
  *
@@ -1841,6 +1974,32 @@ int32_t kcapi_pbkdf(const char *hashname,
  */
 uint32_t kcapi_pbkdf_iteration_count(const char *hashname, uint64_t timeshresh);
 
+/**
+ * kcapi_hkdf() - Extract-and-Expand HKDF (RFC5869)
+ *
+ * @hashname: [in] kernel crypto API name of a keyed hash (e.g. hmac(sha1))
+ * @ikm: [in] Input Keying Material (IKM) -- must be provided
+ * @ikmlen: [in] IKM buffer length -- must be non-zero
+ * @salt: [in] salt buffer -- may be NULL
+ * @saltlen: [in] salt buffer length -- may be zero
+ * @info: [in] info buffer -- may be NULL
+ * @infolen: [in] info buffer length -- may be zero
+ * @dst: [out] Buffer to store the generated key in,
+ * @dlen: [in] Length of the dst buffer. This value defines the number of bytes
+ *	generated by the KDF.
+ *
+ * Perform the key-derivation function according to RFC5869. The input data
+ * is defined in sections 2.2 und 2.3 of RFC5869.
+ *
+ * @return 0 upon success;
+ *	   a negative errno-style error code if an error occurred
+ */
+int32_t kcapi_hkdf(const char *hashname,
+		   const uint8_t *ikm, uint32_t ikmlen,
+		   const uint8_t *salt, uint32_t saltlen,
+		   const uint8_t *info, uint32_t infolen,
+		   uint8_t *dst, uint32_t dlen);
+
 #ifdef __cplusplus
 }
 #endif
